# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

import unittest
import os
import random

from .helper import CaptureOutput
from ..helpers import (
    Dict, print_table, parse_requirements, compare_version, cmp_to_key
)


class DictTests(unittest.TestCase):
    def test_init_with_kwargs(self):
        d = Dict(foo='foo', bar='bar')
        self.assertIn('foo', d)
        self.assertIn('bar', d)

    def test_init_with_zip_list(self):
        d = Dict(zip(('foo', 'bar'), ('bar', 'foo')))
        self.assertIn('foo', d)
        self.assertIn('bar', d)

    def test_init_with_dict(self):
        d = Dict({'foo': 'foo', 'bar': 'bar'})
        self.assertIn('foo', d)
        self.assertIn('bar', d)

    def test_getattr(self):
        d = Dict(foo='bar', bar='foo')
        self.assertEqual(d.foo, 'bar')
        self.assertEqual(d.bar, 'foo')

    def test_attr_error(self):
        d = Dict({'foo': 'bar'})
        with self.assertRaises(AttributeError):
            d.bar


class PrintTableTests(unittest.TestCase):
    def test_default_headers(self):
        rows = [('pigar', '0.5.0', '1.1.1'), ('test', '8.0.0', '2.4.8')]
        target = [
            ' ============================', '  PACKAGE | CURRENT | LATEST',
            '  --------+---------+-------', '  pigar   | 0.5.0   | 1.1.1 ',
            '  test    | 8.0.0   | 2.4.8 ', ' ============================'
        ]
        with CaptureOutput() as output:
            print_table(rows)
        self.assertListEqual(output, target)

    def test_custom_headers(self):
        headers = ['PACKAGE', 'VERSION']
        rows = [('pigar', '1.1.1')]
        target = [
            ' ===================', '  PACKAGE | VERSION',
            '  --------+--------', '  pigar   | 1.1.1  ',
            ' ==================='
        ]
        with CaptureOutput() as output:
            print_table(rows, headers)
        self.assertListEqual(output, target)


class ParseReqsTest(unittest.TestCase):
    def test_parse_requirements(self):
        path = os.path.join(os.path.dirname(__file__), './fake_reqs.txt')
        target = {'a': '4.1.4', 'b': '2.3.0', 'c': ''}
        reqs = parse_requirements(path)
        self.assertDictEqual(reqs, target)


class CompareVersionTests(unittest.TestCase):
    def test_compare_version(self):
        self.assertEqual(compare_version('1.1.1', '1.2.1'), -1)
        self.assertEqual(compare_version('1.10.1', '1.2.1'), 1)
        self.assertEqual(compare_version('1.2', '1.2.1'), -1)
        self.assertEqual(compare_version('1.1.1', '1.1.1'), 0)
        self.assertEqual(compare_version('1.1.1', '1.1.1b'), -1)
        self.assertEqual(compare_version('1.1b2', '1.1a2'), 1)
        self.assertEqual(compare_version('1.1b1', '1.1b2'), -1)
        self.assertEqual(compare_version('1.1b', '1.1b'), 0)

    def test_sort_versions(self):
        target = ['1.1.1', '1.1.1b1', '1.1.1b2', '1.2.1', '1.10.1']
        test = target[:]
        random.shuffle(test)
        self.assertListEqual(
            sorted(test, key=cmp_to_key(compare_version)), target
        )



# -*- coding: utf-8 -*-

import os
import sys
import collections

import foobar

eval('import foo')

exec("import bar", {}, {})

exec("import foobaz")


def foo():
    import json
    pass


class A(object):
    """
    >>> import itertools
    """
    def baz(self):
        """
        >>> import baz
        """


def bar():
    """
    >>> import queue
    >>> def test():
    ...     import bisect
    ...     pass
    """
    pass


__import__('mod', globals(), locals())

importlib.import_module('name')

importlib.import_module('.name', 'pkg.subpkg')



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

import os
import argparse

from .version import version
from .log import enable_pretty_logging
from .core import (
    RequirementsGenerator, check_requirements_latest_versions,
    search_packages_by_names, update_database
)


def _log_level_check(level):
    levels = ('ERROR', 'WARNING', 'INFO', 'DEBUG')
    if level.upper() in levels:
        return level
    raise argparse.ArgumentTypeError(
        '"{0}" not in "{1}"'.format(level, ', '.join(levels))
    )


def _ignore_dirs_check(d):
    ignore = os.path.abspath(d)
    if not os.path.isdir(ignore):
        raise argparse.ArgumentTypeError('"{0}" is not directory.'.format(d))
    return ignore


def _path_check(path):
    path = os.path.abspath(path)
    fdir = os.path.dirname(path)
    if os.path.isdir(fdir):
        return path
    msg = '"{0}" is not a valid file path.'.format(path)
    raise argparse.ArgumentTypeError(msg)


def _projectpath_check(path):
    path = os.path.abspath(path)
    if os.path.isdir(path):
        return path
    msg = '"{0}" is not a valid directory path.'.format(path)
    raise argparse.ArgumentTypeError(msg)


def _comparison_operator_check(op, supported_ops=('==', '~=', '>=')):
    if op in supported_ops:
        return op
    msg = 'invalid operator: {0}, supported operators: {1}'.format(
        op, supported_ops
    )
    raise argparse.ArgumentTypeError(msg)


parser = argparse.ArgumentParser(
    prog='pigar',
    #  usage='%(prog)s [options]',
    description='Python requirements tool -- %(prog)s, it will '
    'do only one thing at each time. Default action is generate'
    ' requirements.txt in current directory.'
)
parser.add_argument(
    '-v',
    '--version',
    action='version',
    version='%(prog)s {0}'.format(version),
    help='show %(prog)s version information and exit'
)
parser.add_argument(
    '-u',
    '--update',
    dest='update_db',
    action='store_true',
    help='update database, use it when %(prog)s failed you, exit when'
    ' action done'
)
parser.add_argument(
    '-s',  # '--search',
    dest='search_names',
    nargs='+',
    metavar='NAME',
    default=[],
    help='search package name by import name, use it if you do not '
    'know import name come from which package, exit when action done'
)
parser.add_argument(
    '-c',  # '--check',
    dest='check_path',
    metavar='PATH',
    nargs='?',
    type=_path_check,
    const=os.getcwd(),
    help='check requirements for the latest version. If file path not '
    'given, search *requirements.txt in current directory, if not found, '
    'generate file requirements.txt, exit when action done'
)
parser.add_argument(
    '-l',  # '--log_level'
    dest='log_level',
    nargs=1,
    type=_log_level_check,
    default=['WARNING'],
    help='show given level log messages, argument can be '
    '(ERROR, WARNING, INFO, DEBUG), case-insensitive'
)
parser.add_argument(
    '-i',  # '--ignore'
    dest='ignores',
    nargs='+',
    metavar='DIR',
    default=[],
    type=_ignore_dirs_check,
    help='given a list of directory to ignore, relative directory, '
    '*used for* -c and default action'
)
parser.add_argument(
    '-p',  # '--path',
    dest='save_path',
    nargs=1,
    type=_path_check,
    default=[os.path.join(os.getcwd(), 'requirements.txt')],
    help='save requirements in given file path, *used for* default action'
)
parser.add_argument(
    '-P',  # '--projectpath',
    dest='project_path',
    nargs=1,
    type=_projectpath_check,
    default=[os.getcwd()],
    help='project path, which is directory, *used for* default action'
)
parser.add_argument(
    '-o',  # '--comparison-operator',
    dest='comparison_operator',
    nargs=1,
    type=_comparison_operator_check,
    default=['=='],
    help='The comparison operator for versions, alternatives: [==, ~=, >=]'
)
parser.add_argument(
    '--without-referenced-comments',
    dest='ref_comments',
    action='store_false',
    help='Omit requirements.txt comments showing the file and line of '
    'each import'
)


def main():
    args = parser.parse_args()

    enable_pretty_logging(log_level=args.log_level[0])

    if args.update_db:
        update_database()
    elif args.check_path:
        check_requirements_latest_versions(
            args.check_path, args.ignores, args.comparison_operator[0],
            args.ref_comments
        )
    elif args.search_names:
        search_packages_by_names(args.search_names)
    else:
        RequirementsGenerator(
            args.project_path[0], args.save_path[0], args.ignores,
            args.comparison_operator[0], args.ref_comments
        ).generate()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

import logging
import logging.handlers

from .helpers import Color

logger = logging.getLogger('pigar')


def enable_pretty_logging(log_level='info', logger=logger):
    logger.setLevel(getattr(logging, log_level.upper()))
    sh = logging.StreamHandler()
    sh.setFormatter(_LogFormatter())
    logger.addHandler(sh)


class _LogFormatter(logging.Formatter):

    FORMAT = '%(asctime)s %(message)s'
    DATE_FORMAT = '%H:%M:%S'
    DATE_COLOR = Color.BLUE
    LEVEL_COLORS = {
        logging.DEBUG: Color.NONE,
        logging.INFO: Color.GREEN,
        logging.WARNING: Color.YELLOW,
        logging.ERROR: Color.RED,
    }

    def __init__(
        self,
        fmt=FORMAT,
        datefmt=DATE_FORMAT,
        datecolor=DATE_COLOR,
        levelcolors=LEVEL_COLORS
    ):
        logging.Formatter.__init__(self, datefmt=datefmt)
        self._fmt = fmt

        self._datecolor = datecolor
        self._levelcolors = levelcolors

    def format(self, record):
        record.asctime = self._datecolor(self.formatTime(record, self.datefmt))

        color = self._levelcolors.get(record.levelno, lambda x: x)
        message = color(record.getMessage())
        if record.exc_info:
            if not record.exc_text:
                record.exc_text = self.formatException(record.exc_info)
        if record.exc_text:
            message += '\n' + record.exc_text
        record.message = message

        formated = self._fmt % record.__dict__
        return formated.replace('\n', '\n    ')



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

import unittest
import os

from ..__main__ import parser

CUR_DIR = os.getcwd()


def parse_args(args):
    args = parser.parse_args(args)
    return (
        args.log_level[0], args.update_db, args.check_path, args.search_names,
        args.ignores, args.save_path[0], args.project_path[0],
        args.comparison_operator[0], args.ref_comments
    )


class CmdTests(unittest.TestCase):
    def setUp(self):
        reqs_path = os.path.join(CUR_DIR, 'requirements.txt')
        self._default_args = [
            'WARNING', False, None, [], [], reqs_path, CUR_DIR, '==', True
        ]

    def tearDown(self):
        del self._default_args

    def test_default(self):
        self.assertListEqual(list(parse_args([])), self._default_args)

    def test_update(self):
        target = self._default_args
        target[1] = True
        self.assertListEqual(list(parse_args(['-u'])), target)
        self.assertListEqual(list(parse_args(['--update'])), target)

    def test_search(self):
        args = ['-s', 'foo', 'bar']
        target = self._default_args
        target[3] = args[1:]
        self.assertListEqual(list(parse_args(args)), target)

    def test_check(self):
        target = self._default_args
        target[2] = CUR_DIR
        self.assertListEqual(list(parse_args(['-c'])), target)
        args = ['-c', os.path.join('..', CUR_DIR)]
        target[2] = args[-1]
        self.assertListEqual(list(parse_args(args)), target)

    def test_log_level(self):
        args = ['-l', 'info']
        target = self._default_args
        target[0] = args[-1]
        self.assertListEqual(list(parse_args(args)), target)

    def test_ignores(self):
        target = self._default_args
        target[4] = [os.path.join(os.path.join('..', CUR_DIR), 'pigar')]

        args = ['-i', 'pigar']
        self.assertListEqual(list(parse_args(args)), target)
        args = ['-i', './pigar']
        self.assertListEqual(list(parse_args(args)), target)
        args = ['-i', 'pigar/']
        self.assertListEqual(list(parse_args(args)), target)

    def test_save_path(self):
        args = ['-p', os.path.join(os.path.join('..', CUR_DIR), 'test.txt')]
        target = self._default_args
        target[5] = args[-1]
        self.assertListEqual(list(parse_args(args)), target)

    def test_project_path(self):
        args = ['-P', os.path.join('..', CUR_DIR)]
        target = self._default_args
        target[6] = args[-1]
        self.assertListEqual(list(parse_args(args)), target)

    def test_comparison_operator(self):
        args = ['-o', '>=']
        target = self._default_args
        target[7] = args[-1]
        self.assertListEqual(list(parse_args(args)), target)
